﻿$ServerCertificateName = "CN=ServiceFabricServerCert"
$ClientCertificateName = "CN=ServiceFabricClientCert"

function IsSecurityX509([string]$ClusterConfigFilePath)
{
    $jsonConfig = Get-Content $ClusterConfigFilePath -Raw | ConvertFrom-Json
    $properties = $jsonConfig.properties
    if ($properties -ne $Null)
    {
        $security = $properties.security
        if ($security -ne $Null )
        {
            $clusterCredentialType = $security.ClusterCredentialType
            return ($clusterCredentialType -ne $Null -And $clusterCredentialType -eq "X509")
        }
    }
    return $False
}

function IsLocalMachine([string] $MachineName)
{
    return ($MachineName -eq $env:ComputerName) -Or ($MachineName -eq "localhost")
}

function GetHostName([string] $iPAddress)
{
    $hostName = $iPAddress
    if ([bool]($iPAddress -as [ipaddress]))
    {
        $hostName = [System.Net.Dns]::GetHostByAddress($iPAddress).HostName
    }

    return $hostName
}

function IsThisMachineServer([string] $ClusterConfigFilePath)
{
    $jsonConfig = Get-Content $ClusterConfigFilePath -Raw | ConvertFrom-Json
    $nodes = $jsonConfig.nodes

    Foreach ($node in $nodes)
    {
        $hostName = GetHostName -iPAddress $node.iPAddress

        if (IsLocalMachine -MachineName $hostName)
        {
            return $True;
        }
    }

    return $False
}

function InstallCertToLocal([string] $CertSetupScript)
{
    invoke-expression "& '$CertSetupScript' -Install -CertSubjectName '$ServerCertificateName'"
    invoke-expression "& '$CertSetupScript' -Install -CertSubjectName '$ClientCertificateName'"

    $cerLoc = "cert:\LocalMachine\My"
    $cert =  Get-ChildItem -Path $cerLoc | where { $_.Subject -like "*$ServerCertificateName*" }
    $serverthumbprint = $cert.Thumbprint

    $cert =  Get-ChildItem -Path $cerLoc | where { $_.Subject -like "*$ClientCertificateName*" }
    $clientthumbprint = $cert.Thumbprint
    
    return $serverthumbprint,$clientthumbprint
}

function IsOneBox([string] $ClusterConfigFilePath)
{
    $jsonConfig = Get-Content $ClusterConfigFilePath -Raw | ConvertFrom-Json
    $nodes = $jsonConfig.nodes

    $machineNames = $nodes.iPAddress | select -uniq
    return ($machineNames.Count -eq 1)
}

function ExportCertificateToLocal([string]$PackageRoot, [string]$CertSetupScript, [string]$ServerThumbprint, [string]$ClientThumbprint, [string]$ClusterConfigFilePath)
{
    $serverCertificate = ExportCertificate -PackageRoot $packageRoot -Thumbprint $serverThumbprint -Name "server"
    $clientCertificate = ExportCertificate -PackageRoot $packageRoot -Thumbprint $clientThumbprint -Name "client"

    Write-Host "Server certificate is exported to $($serverCertificate[0]) with the password $($serverCertificate[1])"
    Write-Host "Client certificate is exported to $($clientCertificate[0]) with the password $($clientCertificate[1])"

    # If the current machine is server, then remove the client cert. If the current machine is client, then remove server cert.
    if (IsThisMachineServer -ClusterConfigFilePath $ClusterConfigFilePath)
    {
        if (-Not (IsOneBox -ClusterConfigFilePath $ClusterConfigFilePath))
        {
            Write-Host "Remove client certificate on localhost."
            invoke-expression "& '$CertSetupScript' -Clean -CertSubjectName '$ClientCertificateName'"
        }
    }
    else 
    {
        Write-Host "Remove server certificate on localhost."
        invoke-expression "& '$CertSetupScript' -Clean -CertSubjectName '$ServerCertificateName'"
    }

    return $serverCertificate
}

function ExportCertificate([string]$PackageRoot, [string]$Thumbprint, [string] $Name)
{
    $OutputCertificateFolder = Join-Path $PackageRoot "Certificates"
    if (-Not(Test-Path $OutputCertificateFolder))
    {
        New-Item -ItemType Directory -Force -Path $OutputCertificateFolder > $null
    }

    $PfxFilePath = "$OutputCertificateFolder\$Name.pfx"
    $randomnum = Get-Random
    $pswd = ConvertTo-SecureString -String "$randomnum" -Force -AsPlainText
    
    Get-ChildItem -Path "cert:\localMachine\my\$Thumbprint" | Export-PfxCertificate -FilePath $PfxFilePath -Password $pswd > $null

    return $PfxFilePath,$randomnum
}

function ModifyJsonThumbprint([string]$ClusterConfigFilePath, [string]$ServerThumbprint, [string]$ClientThumbprint, [string]$OutputPath)
{
    Write-Host "Modify thumbprint in $ClusterConfigFilePath"

    $jsonConfig = Get-Content $ClusterConfigFilePath -Raw | ConvertFrom-Json
    $securityCertInfo = New-Object -TypeName PSObject

    $certificate= New-Object –TypeName PSObject
    $certificate | Add-Member -Name "Thumbprint" -value "$ServerThumbprint" -MemberType NoteProperty
    $certificate | Add-Member -Name "X509StoreName" -value "My" -MemberType NoteProperty

    $clientcertificate = New-Object -TypeName PSObject
    $clientcertificate | Add-Member -Name "CertificateThumbprint" -value "$ClientThumbprint" -MemberType NoteProperty
    $clientcertificate | Add-Member -Name "IsAdmin" -value $true -MemberType NoteProperty
    $clientcertificateArray = @()
    $clientcertificateArray += $clientcertificate

    $securityCertInfo | Add-Member -Name "ClusterCertificate" -value $certificate -MemberType NoteProperty
    $securityCertInfo | Add-Member -Name "ServerCertificate" -value $certificate -MemberType NoteProperty
    $securityCertInfo | Add-Member -Name "ClientCertificateThumbprints" -value $clientcertificateArray -MemberType NoteProperty

    if (IsReverseProxyEndPointConfigured -ClusterConfigFilePath $ClusterConfigFilePath)
    {
        $securityCertInfo | Add-Member -Name "ReverseProxyCertificate" -value $certificate -MemberType NoteProperty
    }

    $jsonConfig.properties.security.CertificateInformation = $securityCertInfo
    $jsonObject = ConvertTo-Json $jsonConfig -Depth 10
    $jsonObject > $OutputPath
}

function IsReverseProxyEndPointConfigured([string]$ClusterConfigFilePath)
{
    $jsonConfig = Get-Content $ClusterConfigFilePath -Raw | ConvertFrom-Json
    $nodes = $jsonConfig.nodes
    $properties = $jsonConfig.properties
    $nodetypes = $properties.nodetypes

    foreach($node in $nodes)
    {
        $nodetypeRef = $node.nodetypeRef
        foreach($nodetype in $nodetypes)
        {
            if ($nodetype.Name -eq $nodetypeRef)
            {
                if ($nodetype.reverseProxyEndpointPort -ne $Null)
                {
                    return $True;
                }
            }
        }
    }

    return $False
}

function AddToTrustedHosts([string]$MachineName)
{
    Write-Host "Adding $MachineName to TrustedHosts"
    Set-Item WSMan:\localhost\Client\TrustedHosts -Value "$MachineName" -Concatenate -Force
}

function InstallCertToRemote([string]$ClusterConfigFilePath, [string]$CertificatePath, [string]$Password)
{
    $jsonConfig = Get-Content $ClusterConfigFilePath -Raw | ConvertFrom-Json
    $nodes = $jsonConfig.nodes
    $pswd = ConvertTo-SecureString -String $Password -Force -AsPlainText
    $certBytes = [System.IO.File]::ReadAllBytes($CertificatePath)

    $iPAddressList = $nodes.iPAddress | select -uniq

    Foreach ($iPAddress in $iPAddressList)
    {
        $machineName = GetHostName -iPAddress $iPAddress

        if (IsLocalMachine -MachineName $machineName)
        {
            Write-Host "Skipping certificate installation in $machineName."
            continue
        }

        AddToTrustedHosts -MachineName $machineName

        Write-Host "Connectting to $machineName"

        for($retry = 0; $retry -le 2; $retry ++)
        {
            Write-Host "Installing server certificate in $machineName for iteration $retry"
            Invoke-Command -ComputerName $machineName -ScriptBlock {
                param($pswd, $certBytes)
                Get-ChildItem -Path Cert:\LocalMachine\My | ? { $_.Subject -like "*$ServerCertificateName*" } | Remove-Item -Force
                Get-ChildItem -Path Cert:\LocalMachine\root | ? { $_.Subject -like "*$ServerCertificateName*" } | Remove-Item -Force
                Get-ChildItem -Path Cert:\CurrentUser\My | ? { $_.Subject -like "*$ServerCertificateName*" } | Remove-Item -Force

                $certPath = [System.IO.Path]::GetTempFileName()
                [system.IO.file]::WriteAllBytes($certPath, $certBytes);

                Import-PfxCertificate -Exportable -CertStoreLocation Cert:\LocalMachine\My -FilePath $certPath -Password $pswd > $null   
                Import-PfxCertificate -Exportable -CertStoreLocation Cert:\LocalMachine\root -FilePath $certPath -Password $pswd > $null  
                Import-PfxCertificate -Exportable -CertStoreLocation Cert:\CurrentUser\My -FilePath $certPath -Password $pswd > $null  

            } -ArgumentList $pswd,$certBytes

            if ($?)
            {
                Write-Host "Installed server certificate in $machineName"
                break
            }

            if ($retry -eq 2)
            {
                Write-Host "Installing server certificate in $machineName failed after 3 attemps"
            }
            else 
            {
                Write-Host "Unable to intall server certificate in $machineName, retry after 30 seconds..."
                Start-Sleep -Seconds 30
            } 
        }        
    }  
}
# SIG # Begin signature block
# MIIjhgYJKoZIhvcNAQcCoIIjdzCCI3MCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBMWq/FMy9sFlja
# GYCxcs+DjnGBtKybUoFEfFjnQujDYaCCDYEwggX/MIID56ADAgECAhMzAAABUZ6N
# j0Bxow5BAAAAAAFRMA0GCSqGSIb3DQEBCwUAMH4xCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25p
# bmcgUENBIDIwMTEwHhcNMTkwNTAyMjEzNzQ2WhcNMjAwNTAyMjEzNzQ2WjB0MQsw
# CQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
# ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMR4wHAYDVQQDExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIB
# AQCVWsaGaUcdNB7xVcNmdfZiVBhYFGcn8KMqxgNIvOZWNH9JYQLuhHhmJ5RWISy1
# oey3zTuxqLbkHAdmbeU8NFMo49Pv71MgIS9IG/EtqwOH7upan+lIq6NOcw5fO6Os
# +12R0Q28MzGn+3y7F2mKDnopVu0sEufy453gxz16M8bAw4+QXuv7+fR9WzRJ2CpU
# 62wQKYiFQMfew6Vh5fuPoXloN3k6+Qlz7zgcT4YRmxzx7jMVpP/uvK6sZcBxQ3Wg
# B/WkyXHgxaY19IAzLq2QiPiX2YryiR5EsYBq35BP7U15DlZtpSs2wIYTkkDBxhPJ
# IDJgowZu5GyhHdqrst3OjkSRAgMBAAGjggF+MIIBejAfBgNVHSUEGDAWBgorBgEE
# AYI3TAgBBggrBgEFBQcDAzAdBgNVHQ4EFgQUV4Iarkq57esagu6FUBb270Zijc8w
# UAYDVR0RBEkwR6RFMEMxKTAnBgNVBAsTIE1pY3Jvc29mdCBPcGVyYXRpb25zIFB1
# ZXJ0byBSaWNvMRYwFAYDVQQFEw0yMzAwMTIrNDU0MTM1MB8GA1UdIwQYMBaAFEhu
# ZOVQBdOCqhc3NyK1bajKdQKVMFQGA1UdHwRNMEswSaBHoEWGQ2h0dHA6Ly93d3cu
# bWljcm9zb2Z0LmNvbS9wa2lvcHMvY3JsL01pY0NvZFNpZ1BDQTIwMTFfMjAxMS0w
# Ny0wOC5jcmwwYQYIKwYBBQUHAQEEVTBTMFEGCCsGAQUFBzAChkVodHRwOi8vd3d3
# Lm1pY3Jvc29mdC5jb20vcGtpb3BzL2NlcnRzL01pY0NvZFNpZ1BDQTIwMTFfMjAx
# MS0wNy0wOC5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAgEAWg+A
# rS4Anq7KrogslIQnoMHSXUPr/RqOIhJX+32ObuY3MFvdlRElbSsSJxrRy/OCCZdS
# se+f2AqQ+F/2aYwBDmUQbeMB8n0pYLZnOPifqe78RBH2fVZsvXxyfizbHubWWoUf
# NW/FJlZlLXwJmF3BoL8E2p09K3hagwz/otcKtQ1+Q4+DaOYXWleqJrJUsnHs9UiL
# crVF0leL/Q1V5bshob2OTlZq0qzSdrMDLWdhyrUOxnZ+ojZ7UdTY4VnCuogbZ9Zs
# 9syJbg7ZUS9SVgYkowRsWv5jV4lbqTD+tG4FzhOwcRQwdb6A8zp2Nnd+s7VdCuYF
# sGgI41ucD8oxVfcAMjF9YX5N2s4mltkqnUe3/htVrnxKKDAwSYliaux2L7gKw+bD
# 1kEZ/5ozLRnJ3jjDkomTrPctokY/KaZ1qub0NUnmOKH+3xUK/plWJK8BOQYuU7gK
# YH7Yy9WSKNlP7pKj6i417+3Na/frInjnBkKRCJ/eYTvBH+s5guezpfQWtU4bNo/j
# 8Qw2vpTQ9w7flhH78Rmwd319+YTmhv7TcxDbWlyteaj4RK2wk3pY1oSz2JPE5PNu
# Nmd9Gmf6oePZgy7Ii9JLLq8SnULV7b+IP0UXRY9q+GdRjM2AEX6msZvvPCIoG0aY
# HQu9wZsKEK2jqvWi8/xdeeeSI9FN6K1w4oVQM4Mwggd6MIIFYqADAgECAgphDpDS
# AAAAAAADMA0GCSqGSIb3DQEBCwUAMIGIMQswCQYDVQQGEwJVUzETMBEGA1UECBMK
# V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
# IENvcnBvcmF0aW9uMTIwMAYDVQQDEylNaWNyb3NvZnQgUm9vdCBDZXJ0aWZpY2F0
# ZSBBdXRob3JpdHkgMjAxMTAeFw0xMTA3MDgyMDU5MDlaFw0yNjA3MDgyMTA5MDla
# MH4xCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdS
# ZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMT
# H01pY3Jvc29mdCBDb2RlIFNpZ25pbmcgUENBIDIwMTEwggIiMA0GCSqGSIb3DQEB
# AQUAA4ICDwAwggIKAoICAQCr8PpyEBwurdhuqoIQTTS68rZYIZ9CGypr6VpQqrgG
# OBoESbp/wwwe3TdrxhLYC/A4wpkGsMg51QEUMULTiQ15ZId+lGAkbK+eSZzpaF7S
# 35tTsgosw6/ZqSuuegmv15ZZymAaBelmdugyUiYSL+erCFDPs0S3XdjELgN1q2jz
# y23zOlyhFvRGuuA4ZKxuZDV4pqBjDy3TQJP4494HDdVceaVJKecNvqATd76UPe/7
# 4ytaEB9NViiienLgEjq3SV7Y7e1DkYPZe7J7hhvZPrGMXeiJT4Qa8qEvWeSQOy2u
# M1jFtz7+MtOzAz2xsq+SOH7SnYAs9U5WkSE1JcM5bmR/U7qcD60ZI4TL9LoDho33
# X/DQUr+MlIe8wCF0JV8YKLbMJyg4JZg5SjbPfLGSrhwjp6lm7GEfauEoSZ1fiOIl
# XdMhSz5SxLVXPyQD8NF6Wy/VI+NwXQ9RRnez+ADhvKwCgl/bwBWzvRvUVUvnOaEP
# 6SNJvBi4RHxF5MHDcnrgcuck379GmcXvwhxX24ON7E1JMKerjt/sW5+v/N2wZuLB
# l4F77dbtS+dJKacTKKanfWeA5opieF+yL4TXV5xcv3coKPHtbcMojyyPQDdPweGF
# RInECUzF1KVDL3SV9274eCBYLBNdYJWaPk8zhNqwiBfenk70lrC8RqBsmNLg1oiM
# CwIDAQABo4IB7TCCAekwEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0OBBYEFEhuZOVQ
# BdOCqhc3NyK1bajKdQKVMBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIAQwBBMAsGA1Ud
# DwQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFHItOgIxkEO5FAVO
# 4eqnxzHRI4k0MFoGA1UdHwRTMFEwT6BNoEuGSWh0dHA6Ly9jcmwubWljcm9zb2Z0
# LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1dDIwMTFfMjAxMV8wM18y
# Mi5jcmwwXgYIKwYBBQUHAQEEUjBQME4GCCsGAQUFBzAChkJodHRwOi8vd3d3Lm1p
# Y3Jvc29mdC5jb20vcGtpL2NlcnRzL01pY1Jvb0NlckF1dDIwMTFfMjAxMV8wM18y
# Mi5jcnQwgZ8GA1UdIASBlzCBlDCBkQYJKwYBBAGCNy4DMIGDMD8GCCsGAQUFBwIB
# FjNodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtpb3BzL2RvY3MvcHJpbWFyeWNw
# cy5odG0wQAYIKwYBBQUHAgIwNB4yIB0ATABlAGcAYQBsAF8AcABvAGwAaQBjAHkA
# XwBzAHQAYQB0AGUAbQBlAG4AdAAuIB0wDQYJKoZIhvcNAQELBQADggIBAGfyhqWY
# 4FR5Gi7T2HRnIpsLlhHhY5KZQpZ90nkMkMFlXy4sPvjDctFtg/6+P+gKyju/R6mj
# 82nbY78iNaWXXWWEkH2LRlBV2AySfNIaSxzzPEKLUtCw/WvjPgcuKZvmPRul1LUd
# d5Q54ulkyUQ9eHoj8xN9ppB0g430yyYCRirCihC7pKkFDJvtaPpoLpWgKj8qa1hJ
# Yx8JaW5amJbkg/TAj/NGK978O9C9Ne9uJa7lryft0N3zDq+ZKJeYTQ49C/IIidYf
# wzIY4vDFLc5bnrRJOQrGCsLGra7lstnbFYhRRVg4MnEnGn+x9Cf43iw6IGmYslmJ
# aG5vp7d0w0AFBqYBKig+gj8TTWYLwLNN9eGPfxxvFX1Fp3blQCplo8NdUmKGwx1j
# NpeG39rz+PIWoZon4c2ll9DuXWNB41sHnIc+BncG0QaxdR8UvmFhtfDcxhsEvt9B
# xw4o7t5lL+yX9qFcltgA1qFGvVnzl6UJS0gQmYAf0AApxbGbpT9Fdx41xtKiop96
# eiL6SJUfq/tHI4D1nvi/a7dLl+LrdXga7Oo3mXkYS//WsyNodeav+vyL6wuA6mk7
# r/ww7QRMjt/fdW1jkT3RnVZOT7+AVyKheBEyIXrvQQqxP/uozKRdwaGIm1dxVk5I
# RcBCyZt2WwqASGv9eZ/BvW1taslScxMNelDNMYIVWzCCFVcCAQEwgZUwfjELMAkG
# A1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQx
# HjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEoMCYGA1UEAxMfTWljcm9z
# b2Z0IENvZGUgU2lnbmluZyBQQ0EgMjAxMQITMwAAAVGejY9AcaMOQQAAAAABUTAN
# BglghkgBZQMEAgEFAKCBrjAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgor
# BgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQg9oqHFlV+
# ATY1csDRTQc3A19JWEvrCRFcmn45WXo8OlgwQgYKKwYBBAGCNwIBDDE0MDKgFIAS
# AE0AaQBjAHIAbwBzAG8AZgB0oRqAGGh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbTAN
# BgkqhkiG9w0BAQEFAASCAQAwjkKHGC56rjaD9oy2jhzb6DlTw5J8lZ5FYtgVs6M5
# KQm0LH47/KDZZQGD4TBRA0eZhJ8K6rS2QzSpLqXKbDZR8JCmNFyCXUGMeNPdR9LQ
# FQd1BcIbRaC1nwP+uuqJhm/2O9W0axdk/Y9ECPnBGrLFKzrjne9QqFoWG3+m5dIf
# qZK2ivXQYC3Qlhe+huqF9CW7v9tjZIKKfpnmJSyS5PiZwD1Wjb8HyvB9fzDajbno
# shw7sVJkWUgLdhWiJTdXw24ZcQBHUm/aK31pn6UeQnh5AlHPMOLrO2hmEluMPeJO
# GQ0yI/G5pndr3eIhIFGsPTo/UVVmWuZDF7huoWt0rAhxoYIS5TCCEuEGCisGAQQB
# gjcDAwExghLRMIISzQYJKoZIhvcNAQcCoIISvjCCEroCAQMxDzANBglghkgBZQME
# AgEFADCCAVEGCyqGSIb3DQEJEAEEoIIBQASCATwwggE4AgEBBgorBgEEAYRZCgMB
# MDEwDQYJYIZIAWUDBAIBBQAEIFmiUITKVri0H6sOnNuZoxTMr5Sd0fnq/3MKXrnR
# 9iqZAgZdELV2l1EYEzIwMTkwNjI2MjM0MDI5Ljk3NlowBIACAfSggdCkgc0wgcox
# CzAJBgNVBAYTAlVTMQswCQYDVQQIEwJXQTEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
# A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMS0wKwYDVQQLEyRNaWNyb3NvZnQg
# SXJlbGFuZCBPcGVyYXRpb25zIExpbWl0ZWQxJjAkBgNVBAsTHVRoYWxlcyBUU1Mg
# RVNOOjE3OUUtNEJCMC04MjQ2MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
# cCBzZXJ2aWNloIIOPDCCBPEwggPZoAMCAQICEzMAAADbqm3jIn80ACUAAAAAANsw
# DQYJKoZIhvcNAQELBQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
# b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3Jh
# dGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwHhcN
# MTgwODIzMjAyNjUzWhcNMTkxMTIzMjAyNjUzWjCByjELMAkGA1UEBhMCVVMxCzAJ
# BgNVBAgTAldBMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQg
# Q29ycG9yYXRpb24xLTArBgNVBAsTJE1pY3Jvc29mdCBJcmVsYW5kIE9wZXJhdGlv
# bnMgTGltaXRlZDEmMCQGA1UECxMdVGhhbGVzIFRTUyBFU046MTc5RS00QkIwLTgy
# NDYxJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1wIHNlcnZpY2UwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCnoZqQDJgki0s2bs39O6xPVFz+Uwqi
# 0RAwzqogMN6WuZEmCnUQMJ0JrwMOJRQNnE3wi4xS/c50t/ifvAlhtospxxCIq1YJ
# A6+pxtfB9vK7xu3Y3aETqh1jZmPMsz92BfpIiL+Uau4H12AmLG/tMht+8YFug30i
# xwTfKCatBmd3O+SeEohlYiBf97aJVD9rPSPlKWUhrDkdpUUMsQYU03Wr764ilEbL
# Uc9zwP/z/6wtPI+RqTX837fUquU4ylD5F70/rkRh6K3WOrHWsQqi476BGGKNjQyi
# wlxDwt5vCwUiu1ldAk0sI9wKiTQv7HobNvot2RdvZHMx9YrTsyUvqk0LAgMBAAGj
# ggEbMIIBFzAdBgNVHQ4EFgQUsrrfHYBcc6bNtu3kHIyt3ipDA0gwHwYDVR0jBBgw
# FoAU1WM6XIoxkPNDe3xGG8UzaFqFbVUwVgYDVR0fBE8wTTBLoEmgR4ZFaHR0cDov
# L2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJvZHVjdHMvTWljVGltU3RhUENB
# XzIwMTAtMDctMDEuY3JsMFoGCCsGAQUFBwEBBE4wTDBKBggrBgEFBQcwAoY+aHR0
# cDovL3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNUaW1TdGFQQ0FfMjAx
# MC0wNy0wMS5jcnQwDAYDVR0TAQH/BAIwADATBgNVHSUEDDAKBggrBgEFBQcDCDAN
# BgkqhkiG9w0BAQsFAAOCAQEAFULCmp29GWB+IkeJJOmBldVebldjUZv6R+yXk1kx
# TFhpIBUB/QCZFORPrhHy4I63UDKvzcnBOECKtHG9B1Z2Dijxp4rxQ4ZmnWR9WgUF
# kH/w5b4shxoSA/X2qp7TzgLZudYwBr56Ox+MRs3e0s5rLwPYYMPc63bIgrZMdbOb
# gj3FRXRABYvqEvTS5NjRQubrTyuiRzjS5BTQ6QQWLL+T3dQ9g8mNST4EobIQ0a/1
# MVeG3DPhBxjCl0ZnrsDAHRGJPt2t6jKKTEbfbGSKyKNx+sNt0Oy0RGapXLgazF8U
# /KTl5VKlH1hYv4976/IaD/BCcxyWUa5Kv3CZOlpUZD0hizCCBnEwggRZoAMCAQIC
# CmEJgSoAAAAAAAIwDQYJKoZIhvcNAQELBQAwgYgxCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xMjAwBgNVBAMTKU1pY3Jvc29mdCBSb290IENlcnRp
# ZmljYXRlIEF1dGhvcml0eSAyMDEwMB4XDTEwMDcwMTIxMzY1NVoXDTI1MDcwMTIx
# NDY1NVowfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
# BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQG
# A1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQCpHQ28dxGKOiDs/BOX9fp/aZRrdFQQ1aUKAIKF
# ++18aEssX8XD5WHCdrc+Zitb8BVTJwQxH0EbGpUdzgkTjnxhMFmxMEQP8WCIhFRD
# DNdNuDgIs0Ldk6zWczBXJoKjRQ3Q6vVHgc2/JGAyWGBG8lhHhjKEHnRhZ5FfgVSx
# z5NMksHEpl3RYRNuKMYa+YaAu99h/EbBJx0kZxJyGiGKr0tkiVBisV39dx898Fd1
# rL2KQk1AUdEPnAY+Z3/1ZsADlkR+79BL/W7lmsqxqPJ6Kgox8NpOBpG2iAg16Hgc
# sOmZzTznL0S6p/TcZL2kAcEgCZN4zfy8wMlEXV4WnAEFTyJNAgMBAAGjggHmMIIB
# 4jAQBgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4EFgQU1WM6XIoxkPNDe3xGG8UzaFqF
# bVUwGQYJKwYBBAGCNxQCBAweCgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMA8GA1Ud
# EwEB/wQFMAMBAf8wHwYDVR0jBBgwFoAU1fZWy4/oolxiaNE9lJBb186aGMQwVgYD
# VR0fBE8wTTBLoEmgR4ZFaHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwv
# cHJvZHVjdHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3JsMFoGCCsGAQUFBwEB
# BE4wTDBKBggrBgEFBQcwAoY+aHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraS9j
# ZXJ0cy9NaWNSb29DZXJBdXRfMjAxMC0wNi0yMy5jcnQwgaAGA1UdIAEB/wSBlTCB
# kjCBjwYJKwYBBAGCNy4DMIGBMD0GCCsGAQUFBwIBFjFodHRwOi8vd3d3Lm1pY3Jv
# c29mdC5jb20vUEtJL2RvY3MvQ1BTL2RlZmF1bHQuaHRtMEAGCCsGAQUFBwICMDQe
# MiAdAEwAZQBnAGEAbABfAFAAbwBsAGkAYwB5AF8AUwB0AGEAdABlAG0AZQBuAHQA
# LiAdMA0GCSqGSIb3DQEBCwUAA4ICAQAH5ohRDeLG4Jg/gXEDPZ2joSFvs+umzPUx
# vs8F4qn++ldtGTCzwsVmyWrf9efweL3HqJ4l4/m87WtUVwgrUYJEEvu5U4zM9GAS
# inbMQEBBm9xcF/9c+V4XNZgkVkt070IQyK+/f8Z/8jd9Wj8c8pl5SpFSAK84Dxf1
# L3mBZdmptWvkx872ynoAb0swRCQiPM/tA6WWj1kpvLb9BOFwnzJKJ/1Vry/+tuWO
# M7tiX5rbV0Dp8c6ZZpCM/2pif93FSguRJuI57BlKcWOdeyFtw5yjojz6f32WapB4
# pm3S4Zz5Hfw42JT0xqUKloakvZ4argRCg7i1gJsiOCC1JeVk7Pf0v35jWSUPei45
# V3aicaoGig+JFrphpxHLmtgOR5qAxdDNp9DvfYPw4TtxCd9ddJgiCGHasFAeb73x
# 4QDf5zEHpJM692VHeOj4qEir995yfmFrb3epgcunCaw5u+zGy9iCtHLNHfS4hQEe
# gPsbiSpUObJb2sgNVZl6h3M7COaYLeqN4DMuEin1wC9UJyH3yKxO2ii4sanblrKn
# QqLJzxlBTeCG+SqaoxFmMNO7dDJL32N79ZmKLxvHIa9Zta7cRDyXUHHXodLFVeNp
# 3lfB0d4wwP3M5k37Db9dT+mdHhk4L7zPWAUu7w2gUDXa7wknHNWzfjUeCLraNtvT
# X4/edIhJEqGCAs4wggI3AgEBMIH4oYHQpIHNMIHKMQswCQYDVQQGEwJVUzELMAkG
# A1UECBMCV0ExEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBD
# b3Jwb3JhdGlvbjEtMCsGA1UECxMkTWljcm9zb2Z0IElyZWxhbmQgT3BlcmF0aW9u
# cyBMaW1pdGVkMSYwJAYDVQQLEx1UaGFsZXMgVFNTIEVTTjoxNzlFLTRCQjAtODI0
# NjElMCMGA1UEAxMcTWljcm9zb2Z0IFRpbWUtU3RhbXAgc2VydmljZaIjCgEBMAcG
# BSsOAwIaAxUAW6UpTq8YrPhwPCRPakRbrW/H2zCggYMwgYCkfjB8MQswCQYDVQQG
# EwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
# A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQg
# VGltZS1TdGFtcCBQQ0EgMjAxMDANBgkqhkiG9w0BAQUFAAIFAOC+f60wIhgPMjAx
# OTA2MjcwNzM1MDlaGA8yMDE5MDYyODA3MzUwOVowdzA9BgorBgEEAYRZCgQBMS8w
# LTAKAgUA4L5/rQIBADAKAgEAAgIRlgIB/zAHAgEAAgIRejAKAgUA4L/RLQIBADA2
# BgorBgEEAYRZCgQCMSgwJjAMBgorBgEEAYRZCgMCoAowCAIBAAIDB6EgoQowCAIB
# AAIDAYagMA0GCSqGSIb3DQEBBQUAA4GBADE8iNTVWlYqwy/xY5PEk7zAlZYD1rcJ
# 1xuRw0iQ+mM/CQZk5N7lMmsSzpeBQp/aJIbVMjKeE1nl1gSdrgHnUtjnfi/6ZyUz
# ufu5syCQTEPPXi0FPW4My5Xy1bAoX9er5MjAvM8QUazDxKJzs58l98iHrekBD4n5
# CoJY1VIi5v8/MYIDDTCCAwkCAQEwgZMwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgT
# Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29m
# dCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENB
# IDIwMTACEzMAAADbqm3jIn80ACUAAAAAANswDQYJYIZIAWUDBAIBBQCgggFKMBoG
# CSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAvBgkqhkiG9w0BCQQxIgQgPeA/8bfZ
# PDCiSCEpb9XfJHlkTKfZ0KoI8EM1pm6I27wwgfoGCyqGSIb3DQEJEAIvMYHqMIHn
# MIHkMIG9BCACUx2x05gBLSK1aiMlY7CLfUe24iC3/LD68uudIYz+qjCBmDCBgKR+
# MHwxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdS
# ZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJjAkBgNVBAMT
# HU1pY3Jvc29mdCBUaW1lLVN0YW1wIFBDQSAyMDEwAhMzAAAA26pt4yJ/NAAlAAAA
# AADbMCIEIEkBXxP7czYS5IjLzUPD1rmkgJdGU1AvtdmHM96EQnAvMA0GCSqGSIb3
# DQEBCwUABIIBAIxHvQaKFPn/iBB1M9BN2gaQJuvtyvGuI1Ti3vPMmachfzXg2Kr4
# vjQd68oLRsqcymdTNoXak7FEff7Nmtoqa+jkB93bmDSNdS+ZiV1bCtFA35uGVMxw
# j6x14hhsdC6kLEEtSJO1yOYWPBMVPjiDF2ic9yEMo9YskKm1ms5XqyMkFfYd3rAZ
# NHQqZtkFbJ+XJwhATQF4WsMntfQ8cNwtCVN0FTYc06rpXcUVvivYthhgQyFUclld
# PyoNZGSVyR+adGTokbGKKL7Sp9FRMUcBZ8MrkuXO6jE3Gveuy9ANhxArwMH/SP9S
# BL1O7A0FhLGxAKttLPzc3eT++tXQ13fE5gY=
# SIG # End signature block
